@@ -288,8 +288,6 @@ class CacheTest(helper.CPWebCase):
         self.getPage("/clear_cache?path=" +
             quote('/long_process?seconds=%d' % SECONDS, safe=''))
         self.assertStatus(200)
-        sys.stdout.write("prepped... ")
-        sys.stdout.flush()
         
         start = datetime.datetime.now()
         def run():
@@ -302,8 +300,9 @@ class CacheTest(helper.CPWebCase):
         for t in ts:
             t.join()
         self.assertEqualDates(start, datetime.datetime.now(),
-                              # Allow a second for our thread/TCP overhead etc.
-                              seconds=SECONDS + 1.1)
+                              # Allow a second (two, for slow hosts)
+                              # for our thread/TCP overhead etc.
+                              seconds=SECONDS + 2)
     
     def test_cache_control(self):
         self.getPage("/control")
