@@ -1,4 +1,5 @@
 import cherrypy
+from cherrypy._cpcompat import ntou
 from cherrypy._cptree import Application
 from cherrypy.test import helper
 
@@ -356,7 +357,7 @@ class ObjectMappingTest(helper.CPWebCase):
         self.assertHeader('Allow', 'GET, HEAD, POST')
         
         self.getPage("/bymethod")
-        self.assertBody("['another', u'one']")
+        self.assertBody(repr(['another', ntou('one')]))
         self.assertHeader('Allow', 'GET, HEAD, POST')
         
         self.getPage("/bymethod", method="PUT")
